@@ -58,8 +58,9 @@ gi.require_version('Gst', '1.0')
 gi.require_version('Wnck', '3.0')
 gi.require_version('SugarExt', '1.0')
 gi.require_version('GdkX11', '3.0')
-gi.require_version('WebKit2', '4.0')
+gi.require_version('GConf', '2.0')
 
+from gi.repository import GObject
 from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import Gtk
@@ -183,11 +184,26 @@ def _check_for_window_manager(screen):
         _complete_desktop_startup()
 
 
-def _start_window_manager():
-    global _metacity_process
+def __window_manager_failed_cb(fd, condition):
+    logging.error('window manager did fail, restarting')
+    GObject.source_remove(_metacity_sid)
+    GObject.timeout_add(1000, _restart_window_manager)
+    return False
+
+
+def _restart_window_manager():
+    global _metacity_process, _metacity_sid
 
     _metacity_process = subprocess.Popen(
-        ['metacity', '--no-force-fullscreen'])
+        ['metacity', '--no-force-fullscreen'],
+        stdout=subprocess.PIPE)
+    _metacity_sid = GObject.io_add_watch(_metacity_process.stdout, GLib.IO_HUP,
+                                         __window_manager_failed_cb)
+    return False
+
+
+def _start_window_manager():
+    _restart_window_manager()
 
     screen = Wnck.Screen.get_default()
     screen.connect('window-manager-changed', __window_manager_changed_cb)
@@ -196,7 +212,6 @@ def _start_window_manager():
 
 
 def _stop_window_manager():
-    global _metacity_process
     _metacity_process.terminate()
 
 
